@@ -13,6 +13,10 @@ use constant BF_LOGIN_ENDPOINT   => 'https://identitysso.betfair.com/api/login/'
 use constant BF_LOGOUT_ENDPOINT  => 'https://identitysso.betfair.com/api/logout/';
 use constant BF_KPALIVE_ENDPOINT => 'https://identitysso.betfair.com/api/keepAlive/';
 use constant BF_ACCOUNT_ENDPOINT => 'https://api.betfair.com/exchange/account/rest/v1.0/';
+use constant BF_HRTBEAT_ENDPOINT => 'https://api.betfair.com/exchange/heartbeat/json-rpc/v1/';
+use constant BF_RSTATUS_ENDPOINT => 'https://api.betfair.com/exchange/scores/json-rpc/v1/';
+use constant AU_BETTING_ENDPOINT => 'https://api-au.betfair.com/exchange/betting/rest/v1.0/';
+use constant AU_ACCOUNT_ENDPOINT => 'https://api-au.betfair.com/exchange/account/rest/v1.0/';
 
 =head1 NAME
 
@@ -20,11 +24,11 @@ WWW::BetfairNG - Object-oriented Perl interface to the Betfair JSON API
 
 =head1 VERSION
 
-Version 0.09
+Version 0.12
 
 =cut
 
-our $VERSION = '0.10';
+our $VERSION = '0.12';
 
 =head1 SYNOPSIS
 
@@ -111,7 +115,10 @@ sub new {
     # set non-configurable attributes
     $self->{error}      = 'OK',
     $self->{response}   = {};
-    $self->{p_check}    =  0;
+    $self->{p_check}    = 0;
+    $self->{australian} = 0;
+    $self->{bet_end_pt} = BF_BETTING_ENDPOINT;
+    $self->{acc_end_pt} = BF_ACCOUNT_ENDPOINT;
     $self->{data_types} = {};
     # Create an HTTP::Tiny object to do all the heavy lifting
     my $client = HTTP::Tiny->new(
@@ -239,6 +246,38 @@ sub check_parameters {
   return $self->{p_check};
 }
 
+=head3 australian()
+
+  my $is_aus = $bf->australian();
+  $bf->australian('<boolean>');
+
+Gets or sets a flag telling the object whether or not it should be talking to the Australian
+exchange. If this is set, all calls to Betting and Accounts Operations will be directed to
+the Australian exchange, which is a seperate entity hosted in Australia as required by
+Australian Government legislation. All bets on Australian horseracing markets must be made
+through this exchange, and a seperate 'wallet' is required to hold the stakes and returns.
+The default value is 'false', meaning that calls are directed to the UK exchange. Changing
+the value of 'australian' does NOT require you to log out and log back in, as the current
+session is maintained (in particular, 'keepAlive' still needs to be called with the same
+frequency). Currently, setting up a 'heartbeat' service will not affect bets placed on the
+Australian exchange, as this has not been implemented by Betfair.
+
+=cut
+
+sub australian {
+  my $self = shift;
+  if (@_){
+    my $current_state = $self->{australian};
+    my $flag = shift;
+    $self->{australian} = $flag ? 1 : 0;
+    unless ($self->{australian} == $current_state) {
+      $self->{bet_end_pt} = $self->{australian} ? AU_BETTING_ENDPOINT : BF_BETTING_ENDPOINT;
+      $self->{acc_end_pt} = $self->{australian} ? AU_ACCOUNT_ENDPOINT : BF_ACCOUNT_ENDPOINT;
+    }
+  }
+  return $self->{australian};
+}
+
 =head3 error()
 
   my $err_str = $bf->error();
@@ -294,17 +333,19 @@ sub response {
 =head1 API CALLS
 
 These are generally of the form '$return_value = $bf->someCall($parameters)', where
-'$parameters' is a hash reference (or anonymous hash) containing one or more BETFAIR DATA
-TYPES (described below), and $return_value is a hash or array reference, again containing
-one or more BETFAIR DATA TYPES. Many of these data types are straightforward lists or
-hashes of scalars, but some are quite complex structures. Depending on the call, some
-parameters may be required (RQD) and others may be optional (OPT). A check for the
-existence of required parameters is made before a call is despatched, but no detailed
-checking is made of the data structures to make sure they conform to what Betfair expects
-- the call is just sent 'as is' to the API, and any errors in the construction of the hash
-will result in an error being returned by Betfair. Any error in a call, for whatever
-reason, will result in a $return_value of '0'. In this case, $bf->error() will contain a
-string describing the error and further details of the error may be found by examining
+'$parameters' is a hash reference (or anonymous hash) containing one or more BETFAIR
+DATA TYPES (described below), and $return_value is a hash or array reference, again
+containing one or more BETFAIR DATA TYPES. Many of these data types are straightforward
+lists or hashes of scalars, but some are quite complex structures. Depending on the
+call, some parameters may be required (RQD) and others may be optional (OPT). If
+$bf->check_parameters() is set to 'true', the parameter hash will be checked before it
+is sent to the API, and any errors in construction will result in the method call
+immediately returning '0' and C<< $bf->error >> being set to a message detailing the
+precise problem. If $bf->check_parameters() is set to 'false' (the default), the
+parameter hash is sent 'as is' to Betfair, and any problems with it's construction will
+result in a Betfair error message.  Any error in a call, for whatever reason, will
+result in a $return_value of '0'. In this case, $bf->error() will contain a string
+describing the error and further details of the error may be found by examining
 $bf->response().
 
 
@@ -471,7 +512,7 @@ sub logout {
   my $return_value = $bf->keepAlive();
 
 Sends a 'Keep Alive' message to the host. Without this, the session will time out after
-about twelve hours. Unlike the SOAP interface, other API calls do NOT reset the timeout;
+about four hours. Unlike the SOAP interface, other API calls do NOT reset the timeout;
 it has to be done explicitly with a 'keepAlive'. Returns '1' if the keepAlive succeeded,
 '0' if any errors were encountered.
 
@@ -538,7 +579,7 @@ Return Value
 sub listCompetitions {
   my $self = shift;
   my $params = shift || {};
-  my $url = BF_BETTING_ENDPOINT.'listCompetitions/';
+  my $url = $self->{bet_end_pt}.'listCompetitions/';
   my $result = $self->_callAPI($url, $params);
   return $result;
 }
@@ -563,7 +604,7 @@ Return Value
 sub listCountries {
   my $self = shift;
   my $params = shift || {};
-  my $url = BF_BETTING_ENDPOINT.'listCountries/';
+  my $url = $self->{bet_end_pt}.'listCountries/';
   my $result = $self->_callAPI($url, $params);
   return $result;
 }
@@ -599,7 +640,7 @@ Return Value
 sub listCurrentOrders {
   my $self = shift;
   my $params = shift || {};
-  my $url = BF_BETTING_ENDPOINT.'listCurrentOrders/';
+  my $url = $self->{bet_end_pt}.'listCurrentOrders/';
   my $result = $self->_callAPI($url, $params);
   return $result;
 }
@@ -639,7 +680,7 @@ Return Value
 sub listClearedOrders {
   my $self = shift;
   my $params = shift || {};
-  my $url = BF_BETTING_ENDPOINT.'listClearedOrders/';
+  my $url = $self->{bet_end_pt}.'listClearedOrders/';
   my $result = $self->_callAPI($url, $params);
   return $result;
 }
@@ -664,7 +705,7 @@ Return Value
 sub listEvents {
   my $self = shift;
   my $params = shift || {};
-  my $url = BF_BETTING_ENDPOINT.'listEvents/';
+  my $url = $self->{bet_end_pt}.'listEvents/';
   my $result = $self->_callAPI($url, $params);
   return $result;
 }
@@ -690,7 +731,7 @@ Return Value
 sub listEventTypes {
   my $self = shift;
   my $params = shift || {};
-  my $url = BF_BETTING_ENDPOINT.'listEventTypes/';
+  my $url = $self->{bet_end_pt}.'listEventTypes/';
   my $result = $self->_callAPI($url, $params);
   return $result;
 }
@@ -722,7 +763,7 @@ Return Value
 sub listMarketBook {
   my $self = shift;
   my $params = shift || {};
-  my $url = BF_BETTING_ENDPOINT.'listMarketBook/';
+  my $url = $self->{bet_end_pt}.'listMarketBook/';
   my $result = $self->_callAPI($url, $params);
   return $result;
 }
@@ -753,7 +794,7 @@ Return Value
 sub listMarketCatalogue {
   my $self = shift;
   my $params = shift || {};
-  my $url = BF_BETTING_ENDPOINT.'listMarketCatalogue/';
+  my $url = $self->{bet_end_pt}.'listMarketCatalogue/';
   my $result = $self->_callAPI($url, $params);
   return $result;
 }
@@ -782,7 +823,7 @@ Return Value
 sub listMarketProfitAndLoss {
   my $self = shift;
   my $params = shift || {};
-  my $url = BF_BETTING_ENDPOINT.'listMarketProfitAndLoss/';
+  my $url = $self->{bet_end_pt}.'listMarketProfitAndLoss/';
   my $result = $self->_callAPI($url, $params);
   return $result;
 }
@@ -808,7 +849,7 @@ Return Value
 sub listMarketTypes {
   my $self = shift;
   my $params = shift || {};
-  my $url = BF_BETTING_ENDPOINT.'listMarketTypes/';
+  my $url = $self->{bet_end_pt}.'listMarketTypes/';
   my $result = $self->_callAPI($url, $params);
   return $result;
 }
@@ -834,7 +875,7 @@ Return Value
 sub listTimeRanges {
   my $self = shift;
   my $params = shift || {};
-  my $url = BF_BETTING_ENDPOINT.'listTimeRanges/';
+  my $url = $self->{bet_end_pt}.'listTimeRanges/';
   my $result = $self->_callAPI($url, $params);
   return $result;
 }
@@ -861,7 +902,7 @@ Return Value
 sub listVenues {
   my $self = shift;
   my $params = shift || {};
-  my $url = BF_BETTING_ENDPOINT.'listVenues/';
+  my $url = $self->{bet_end_pt}.'listVenues/';
   my $result = $self->_callAPI($url, $params);
   return $result;
 }
@@ -905,7 +946,7 @@ Return Value
 sub placeOrders {
   my $self = shift;
   my $params = shift || {};
-  my $url = BF_BETTING_ENDPOINT.'placeOrders/';
+  my $url = $self->{bet_end_pt}.'placeOrders/';
   my $result = $self->_callAPI($url, $params);
   if ($result) {
     my $status = $result->{status};
@@ -947,7 +988,7 @@ Return Value
 sub cancelOrders {
   my $self = shift;
   my $params = shift || {};
-  my $url = BF_BETTING_ENDPOINT.'cancelOrders/';
+  my $url = $self->{bet_end_pt}.'cancelOrders/';
   my $result = $self->_callAPI($url, $params);
   return $result;
 }
@@ -987,7 +1028,7 @@ Return Value
 sub replaceOrders {
   my $self = shift;
   my $params = shift || {};
-  my $url = BF_BETTING_ENDPOINT.'replaceOrders/';
+  my $url = $self->{bet_end_pt}.'replaceOrders/';
   my $result = $self->_callAPI($url, $params);
   if ($result) {
     my $status = $result->{status};
@@ -1032,7 +1073,7 @@ Return Value
 sub updateOrders {
   my $self = shift;
   my $params = shift || {};
-  my $url = BF_BETTING_ENDPOINT.'updateOrders/';
+  my $url = $self->{bet_end_pt}.'updateOrders/';
   my $result = $self->_callAPI($url, $params);
   if ($result) {
     my $status = $result->{status};
@@ -1051,12 +1092,12 @@ sub updateOrders {
 
 As with the Betting Operations, the descriptions of these methods are taken directly from
 the Betfair documentation. Once again, required parameters are denoted by RQD and optional
-ones by OPT. Some parameters are described in terms of BETFAIR FATA TYPES, which are
+ones by OPT. Some parameters are described in terms of BETFAIR DATA TYPES, which are
 described below.
 
 =head3 createDeveloperAppKeys($parameters)
 
-  my $return_value = createDeveloperAppKeys(<application name>);
+  my $return_value = $bf->createDeveloperAppKeys(<application name>);
 
 Create two application keys for given user; one active and the other delayed. NOTE as this
 call fails if the keys have already been created, it has NOT BEEN TESTED.
@@ -1076,14 +1117,14 @@ Return Value
 sub createDeveloperAppKeys {
   my $self = shift;
   my $params = shift || {};
-  my $url = BF_ACCOUNT_ENDPOINT.'createDeveloperAppKeys/';
+  my $url = $self->{acc_end_pt}.'createDeveloperAppKeys/';
   my $result = $self->_callAPI($url, $params);
   return $result;
 }
 
 =head3 getAccountDetails()
 
-  my $return_value = getAccountDetails();
+  my $return_value = $bf->getAccountDetails();
 
 Returns the details relating [to] your account, including your discount rate and Betfair
 point balance. Takes no parameters.
@@ -1098,20 +1139,21 @@ Return Value
   timezone          String
   discountRate      Double
   pointsBalance     Integer
+  countryCode       String
 
 =cut
 
 sub getAccountDetails {
   my $self = shift;
   my $params = shift || {};
-  my $url = BF_ACCOUNT_ENDPOINT.'getAccountDetails/';
+  my $url = $self->{acc_end_pt}.'getAccountDetails/';
   my $result = $self->_callAPI($url, $params);
   return $result;
 }
 
 =head3 getAccountFunds()
 
-  my $return_value = getAccountFunds([$parameters]);
+  my $return_value = $bf->getAccountFunds([$parameters]);
 
 Get available to bet amount. The getAccounts service will return the
 UK wallet balance by default from either the UK or AUS Accounts API
@@ -1135,14 +1177,14 @@ Return Value
 sub getAccountFunds {
   my $self = shift;
   my $params = shift || {};
-  my $url = BF_ACCOUNT_ENDPOINT.'getAccountFunds/';
+  my $url = $self->{acc_end_pt}.'getAccountFunds/';
   my $result = $self->_callAPI($url, $params);
   return $result;
 }
 
 =head3 getDeveloperAppKeys()
 
-  my $return_value = getDeveloperAppKeys();
+  my $return_value = $bf->getDeveloperAppKeys();
 
 Get all application keys owned by the given developer/vendor. Takes no parameters.
 
@@ -1155,14 +1197,14 @@ Return Value
 sub getDeveloperAppKeys {
   my $self = shift;
   my $params = shift || {};
-  my $url = BF_ACCOUNT_ENDPOINT.'getDeveloperAppKeys/';
+  my $url = $self->{acc_end_pt}.'getDeveloperAppKeys/';
   my $result = $self->_callAPI($url, $params);
   return $result;
 }
 
 =head3 getAccountStatement([$parameters])
 
-  my $return_value = getAccountStatement();
+  my $return_value = $bf->getAccountStatement();
 
 Get Account Statement.
 
@@ -1185,14 +1227,14 @@ Return Value
 sub getAccountStatement {
   my $self = shift;
   my $params = shift || {};
-  my $url = BF_ACCOUNT_ENDPOINT.'getAccountStatement/';
+  my $url = $self->{acc_end_pt}.'getAccountStatement/';
   my $result = $self->_callAPI($url, $params);
   return $result;
 }
 
 =head3 listCurrencyRates([$parameters])
 
-  my $return_value = listCurrencyRates();
+  my $return_value = $bf->listCurrencyRates();
 
 Returns a list of currency rates based on given currency.
 
@@ -1209,16 +1251,16 @@ Return Value
 sub listCurrencyRates {
   my $self = shift;
   my $params = shift || {};
-  my $url = BF_ACCOUNT_ENDPOINT.'listCurrencyRates/';
+  my $url = $self->{acc_end_pt}.'listCurrencyRates/';
   my $result = $self->_callAPI($url, $params);
   return $result;
 }
 
 =head3 transferFunds($parameters)
 
-  my $return_value = transferFunds({from   => 'UK',
-                                    to     => 'AUSTRALIAN',
-                                    amount => <amount> });
+  my $return_value = $bf->transferFunds({from   => 'UK',
+                                         to     => 'AUSTRALIAN',
+                                         amount => <amount> });
 
 Transfer funds between the UK Exchange and Australian Exchange
 wallets. You require funds in the Australian Exchange wallet to bet on
@@ -1239,7 +1281,7 @@ Return Value
 sub transferFunds {
   my $self = shift;
   my $params = shift || {};
-  my $url = BF_ACCOUNT_ENDPOINT.'transferFunds/';
+  my $url = $self->{acc_end_pt}.'transferFunds/';
   my $result = $self->_callAPI($url, $params);
   return $result;
 }
@@ -1331,14 +1373,24 @@ sub navigationMenu {
     $self->{error} = 'No application key set';
     return 0;
   }
-  my $url = BF_BETTING_ENDPOINT.'en/navigation/menu.json';
+  # Can't use default client because we need a longer timeout
+  my $client = HTTP::Tiny->new(
+                     timeout         => 30,
+                     agent           => "WWW::BetfairNG/$VERSION",
+		     verify_SSL      => 1,
+                     default_headers => {'Content-Type'    => 'application/json',
+			                 'Accept'          => 'application/json',
+			                 'Accept-Encoding' => 'gzip'
+			                }
+  );
+  my $url = $self->{bet_end_pt}.'en/navigation/menu.json';
   my $options = {
 		 headers => {
 			     'X-Authentication' => $self->session,
 			     'X-Application'    => $self->app_key
 			    }
 		};
-  my $response = $self->{client}->get($url, $options);
+  my $response = $client->get($url, $options);
   unless ($response->{success}) {
     $self->{error}  = $response->{status}.' '.$response->{reason}.' '.$response->{content};
     return 0;
@@ -1349,6 +1401,86 @@ sub navigationMenu {
   return $self->response;
 }
 
+=head2 Heartbeat API
+
+This Heartbeat operation is provided to allow customers to automatically cancel their
+unmatched bets in the event of their API client losing connectivity with the Betfair API.
+The Heartbeat is not currently available for the Australian exchange.
+
+=head3 heartbeat($parameters)
+
+  my $return_value = $bf->heartbeat({preferredTimeoutSeconds => <timeout>});
+
+This heartbeat operation is provided to help customers have their positions managed
+automatically in the event of their API clients losing connectivity with the Betfair
+API. If a heartbeat request is not received within a prescribed time period, then Betfair
+will attempt to cancel all 'LIMIT' type bets for the given customer on the given
+exchange. There is no guarantee that this service will result in all bets being cancelled
+as there are a number of circumstances where bets are unable to be cancelled. Manual
+intervention is strongly advised in the event of a loss of connectivity to ensure that
+positions are correctly managed. If this service becomes unavailable for any reason, then
+your heartbeat will be unregistered automatically to avoid bets being inadvertently
+cancelled upon resumption of service. you should manage your position manually until the
+service is resumed. Heartbeat data may also be lost in the unlikely event of nodes failing
+within the cluster, which may result in your position not being managed until a subsequent
+heartbeat request is received.
+
+Parameters
+
+  preferredTimeoutSeconds  Integer      RQD
+
+Return Value
+
+  actionPerformed          ActionPerformed
+  actualTimeoutSeconds     Integer
+
+=cut
+
+sub heartbeat {
+  my $self = shift;
+  my $params = shift || {};
+  my $url = BF_HRTBEAT_ENDPOINT;
+  my $action = 'HeartbeatAPING/v1.0/heartbeat';
+  my $result = $self->_callRPC($url, $action, $params);
+  return $result;
+}
+
+=head2 Race Status API
+
+The listRaceDetails operation is provided to allow customers to establish the status of a
+horse race market both prior to and after the start of the race.  This information is
+available for UK and Ireland races only.
+
+=head3 listRaceDetails($parameters)
+
+  my $return_value = $bf->listRaceDetails();
+
+Search for races to get their details. 'meetingIds' optionally restricts the results to
+the specified meeting IDs. The unique Id for the meeting equivalent to the eventId for
+that specific race as returned by listEvents. 'raceIds' optionally restricts the results
+to the specified race IDs. The unique Id for the race in the format meetingid.raceTime
+(hhmm). raceTime is in UTC.
+
+Parameters
+
+  meetingIds     Array of Strings     OPT
+  raceIds        Array of Strings     OPT
+
+Return Value
+
+  ArrayRef       RaceDetails
+
+=cut
+
+sub listRaceDetails {
+  my $self = shift;
+  my $params = shift || {};
+  my $url = BF_RSTATUS_ENDPOINT;
+  my $action = 'ScoresAPING/v1.0/listRaceDetails';
+  my $result = $self->_callRPC($url, $action, $params);
+  return $result;
+}
+
 #===============================#
 # Private Methods and Functions #
 #===============================#
@@ -1403,6 +1535,70 @@ sub _callAPI {
   return $self->{response};
 }
 
+# Called by Heartbeat and Race Status methods to do the talking to Betfair.
+# =========================================================================#
+#                                                                          #
+# (Betfair generally supports both a JSON-REST and a JSON-RPC interface to #
+# the API, and we use REST. However, Heartbeat and Race Status only allow  #
+# RPC, so we need a function for that as well.                             #
+#                                                                          #
+# =========================================================================#
+sub _callRPC {
+  my ($self, $url, $action, $params) = @_;
+  unless ($self->session){
+    $self->{error} = 'Not logged in';
+    return 0;
+  }
+  unless ($self->app_key){
+    $self->{error} = 'No application key set';
+    return 0;
+  }
+    unless(ref($params) eq 'HASH') {
+    $self->{error} = 'Parameters must be a hash ref or anonymous hash';
+    return 0;
+  }
+  if ($self->check_parameters) {
+    my ($method_name) = $action =~ /\/(\w+)$/;
+    return 0 unless $self->_check_parameter($method_name, $params);
+  }
+  my $post    = { params => $params, jsonrpc => "2.0", method => $action, id => 1};
+  my $options = {
+		 headers => {
+			     'X-Authentication' => $self->session,
+			     'X-Application'    => $self->app_key,
+			    },
+		 content => encode_json($post)
+		};
+  my $response = $self->{client}->post($url, $options);
+  unless ($response->{success}) {
+    if ($response->{status} == 400) {
+      my $content = $self->_gunzip($response->{content});
+      $self->{response} = decode_json($content);
+      $self->{error}  = $self->{response}->{detail}->{APINGException}->{errorCode} ||
+	$response->{status}.' '.$response->{reason}.' '.$response->{content};
+    }
+    else {
+      $self->{error}  = $response->{status}.' '.$response->{reason}.' '
+                       .$response->{content};
+    }
+    return 0;
+  }
+  my $content = $self->_gunzip($response->{content});
+  return 0 unless ($content);
+  $self->{response} = decode_json($content);
+  if ($self->{response}->{error}) {
+    $self->{error} = $self->{response}->{error}->{message};
+    return 0;
+  }
+  if ($self->{response}->{result}) {
+    return $self->{response}->{result};
+  }
+  else {
+    $self->{error} = "Empty reply";
+    return 0;
+  }
+}
+
 # HTTP::Tiny doesn't have built-in decompression so we do it here
 # ===============================================================
 sub _gunzip {
@@ -1661,8 +1857,8 @@ sub _load_data_types {
 		       },
   cancelOrders      => {
 			type     => 'HASH',
-			required => [qw/marketId instructions/],
-			allowed  => [qw/customerRef/],
+			required => [qw//],
+			allowed  => [qw/marketId instructions customerRef/],
 		       },
   replaceOrders     => {
 			type     => 'HASH',
@@ -1710,6 +1906,19 @@ sub _load_data_types {
 			required => [qw/from to amount/],
 			allowed  => [qw//],
 		       },
+
+  heartbeat         => {
+			type     => 'HASH',
+			required => [qw/preferredTimeoutSeconds/],
+			allowed  => [qw//],
+		       },
+
+  listRaceDetails   => {
+			type     => 'HASH',
+			required => [qw//],
+			allowed  => [qw/meetingIds raceIds/],
+		       },
+
 # arrays
   betIds            => {
 			type     => 'ARRAY',
@@ -1779,6 +1988,14 @@ sub _load_data_types {
 			type     => 'ARRAY',
                         array_of => 'OrderStatus',
 		       },
+  meetingIds        => {
+			type     => 'ARRAY',
+                        array_of => 'meetingId',
+		       },
+  raceIds           => {
+			type     => 'ARRAY',
+                        array_of => 'raceId',
+		       },
   };
 
 # Common scalars
@@ -1810,7 +2027,7 @@ sub _load_data_types {
 			  };
   $type_defs->{runnerId}  = {
 	       type     => 'SCALAR',
-	       allowed  => qr/^\d{6,10}$/,
+	       allowed  => qr/^\d{3,10}$/,
 	       example  => '6750999'
 			  };
   $type_defs->{currencyCode}  = {
@@ -1824,6 +2041,17 @@ sub _load_data_types {
 	       allowed  => qr/^\d{1,10}$/,
 	       example  => '409999'
 			  };
+  $type_defs->{preferredTimeoutSeconds}  = {
+	       type     => 'SCALAR',
+	       allowed  => qr/^\d{1,3}$/,
+	       example  => '180'
+			  };
+  $type_defs->{meetingId} = $type_defs->{eventId};
+  $type_defs->{raceId}  = {
+	       type     => 'SCALAR',
+	       allowed  => qr/^\d{8,10}\.(0[0-9]|1[0-9]|2[0-3])[0-5][0-9]$/,
+	       example  => '27292599.1430'
+			  };
 
 # betfair data types (all the following pod is still inside the _load_data_types sub)
 # each type and any sub-types are loaded into the hash following their pod entry.
@@ -1839,6 +2067,17 @@ these are denoted by RQD and OPT respectively. Simple scalar type definitions (L
 Double, Integer, String, Boolean, Date) have been retained for convenience. 'Date' is
 a string in ISO 8601 format (e.g. '2007-04-05T14:30Z').
 
+=head3 ActionPerformed
+
+Enumeration
+
+  NONE                              No action was performed since last heartbeat
+  CANCELLATION_REQUEST_SUBMITTED    A request to cancel all unmatched bets was submitted
+  ALL_BETS_CANCELLED                All unmatched bets were cancelled since last heartbeat
+  SOME_BETS_NOT_CANCELLED           Not all unmatched bets were cancelled
+  CANCELLATION_REQUEST_ERROR        There was an error requesting cancellation
+  CANCELLATION_STATUS_UNKNOWN       There was no response from requesting cancellation
+
 =head3 BetStatus
 
 Enumeration
@@ -1900,6 +2139,8 @@ Enumeration
   sizeSettled       Double
   profit            Double
   sizeCancelled     Double
+  lastMatchedDate   Date
+  betOutcome        String
 
 =head3 Competition
 
@@ -2119,6 +2360,17 @@ Enumeration
 
   UNKNOWN     Statement item not mapped to a specific class.
 
+=head3 ItemDescription
+
+  eventTypeDesc     String
+  eventDesc         String
+  marketDesc        String
+  marketStartTime   Date
+  runnerDesc        String
+  numberOfWinners   Integer
+  marketType        String
+  eachWayDivisor    Double
+
 =head3 LimitOnCloseOrder
 
   liability         Double              REQ
@@ -2218,6 +2470,7 @@ Enumeration
   wallet             String
   rules              String
   rulesHasDate       Boolean
+  eachWayDivisor     Double
   clarifications     String
 
 =head3 MarketFilter
@@ -2546,6 +2799,34 @@ Enumeration
 			  };
   $type_defs->{newPrice} = $double;
 
+=head3 RaceDetails
+
+  meetingId         String
+  raceId            String
+  raceStatus        RaceStatus
+  lastUpdated       Date
+  responseCode      ResponseCode
+
+=head3 RaceStatus
+
+Enumeration
+
+  DORMANT           There is no data available for this race
+  DELAYED           The start of the race has been delayed
+  PARADING          The horses are in the parade ring
+  GOINGDOWN         The horses are going down to the starting post
+  GOINGBEHIND       The horses are going behind the stalls
+  ATTHEPOST         The horses are at the post
+  UNDERORDERS       The horses are loaded into the stalls/race is about to start
+  OFF               The race has started
+  FINISHED          The race has finished
+  FALSESTART        There has been a false start
+  PHOTOGRAPH        The result of the race is subject to a photo finish
+  RESULT            The result of the race has been announced
+  WEIGHEDIN         The jockeys have weighed in
+  RACEVOID          The race has been declared void
+  ABANDONED         The meeting has been cancelled
+
 =head3 ReplaceInstructionReport
 
   status                  InstructionReportStatus
@@ -2553,6 +2834,17 @@ Enumeration
   cancelInstructionReport CancelInstructionReport
   placeInstructionReport  PlaceInstructionReport
 
+=head3 ResponseCode
+
+Enumeration
+
+  OK                                  Data returned successfully
+  NO_NEW_UPDATES                      No updates since the passes UpdateSequence
+  NO_LIVE_DATA_AVAILABLE              Event scores are no longer available
+  SERVICE_UNAVAILABLE                 Data feed for the event type is currently unavailable
+  UNEXPECTED_ERROR                    An unexpected error occurred retrieving score data
+  LIVE_DATA_TEMPORARILY_UNAVAILABLE   Live Data feed is temporarily unavailable
+
 =head3 RollupModel
 
 Enumeration
@@ -2778,9 +3070,62 @@ Enumeration
 
 1;
 
+=head1 THREADS
 
+Because the betfair object maintains a persistent encrypted connection to the Betfair
+servers, it should NOT be considered 100% thread-safe. In particular, using the same $bf
+object to make API calls across different threads will usually result in disaster.
+In practice, there are at least two ways to solve this problem and use WWW::BetfairNG
+safely in threaded applications:-
 
+=head2 'Postbox' Thread
 
+If simultaneous or overlapping calls to betfair are not required, one solution is to make
+all calls from a single, dedicated thread. This thread can wait on a queue created by
+Thread::Queue for requests from other threads, and return the result to them, again
+via a queue. Only one $bf object is required in this scenario, which may be created by
+the 'postbox' thread itself or, less robustly, by the parent thread before the 'postbox'
+is spawned. In the latter case, no other thread (including the parent) should use the $bf
+object once the 'postbox' has started using it.
+
+=head2 Multiple Objects
+
+If you need to make simultaneous or overlapping calls to betfair, you can create a new $bf
+object in each thread that makes betfair calls. As betfair sessions are identified by a
+simple scalar session token, a single login will create a session which CAN be safely
+shared across threads:-
+
+  use WWW::BetfairNG;
+  use threads;
+  use threads::shared;
+
+  my $parent_bf = WWW::BetfairNG->new({
+                                       ssl_cert => '<path to ssl certificate file>',
+                                       ssl_key  => '<path to ssl key file>',
+                                       app_key  => '<application key value>',
+                                       });
+  $parent_bf->login({username => <username>, password => <password>})
+    or die;
+  my $session :shared = $parent_bf->session();
+
+  my $child = threads->create(sub {
+    # Create a new bf object in the child - no need for ssl cert and key
+    my $child_bf = WWW::BetfairNG->new({app_key  => '<application key value>'});
+    # Assign the shared session token - $child_bf will then be logged in
+    $child_bf->session($session);
+
+        # Make any required API calls in the child using $child_bf
+  });
+
+  # Freely make API calls using $parent_bf
+
+  $child->join;
+  $parent_bf->logout; # Logs out any children using the same session token
+  exit 0;
+
+In particular, keepAlive calls only need to be made in one thread to affect all threads
+using the same session token, and logging out in any thread will log out all threads
+using the same session token.
 
 =head1 SEE ALSO
 
@@ -2795,11 +3140,13 @@ Myrddin Wyllt, E<lt>myrddinwyllt@tiscali.co.ukE<gt>
 
 Main inspiration for this was David Farrell's WWW::betfair module,
 which was written for the v6 SOAP interface. Thanks also to Carl
-O'Rourke for suggestions on clarifying error messages.
+O'Rourke for suggestions on clarifying error messages and Colin
+Magee for the suggestion to extend the timeout period for the
+navigationMenu call.
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2014 by Myrddin Wyllt
+Copyright (C) 2015 by Myrddin Wyllt
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
